package ch.polgrabia.demos.client;

import io.vertx.core.AbstractVerticle;
import io.vertx.core.AsyncResult;
import io.vertx.core.http.HttpClient;
import io.vertx.core.http.WebSocket;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayOutputStream;

public class WebsocketClientParticle extends AbstractVerticle {
    private static final Logger logger = LoggerFactory.getLogger(WebsocketClientParticle.class);
    private final String hostname;
    private final int port;
    private final String path;
    private HttpClient client;
    private boolean isShuttingDownActivated = false;
    private final Object runningMonitor = new Object();
    private ByteArrayOutputStream clientTextInput;

    public WebsocketClientParticle(String hostname, int port, String path) {
        this.hostname = hostname;
        this.port = port;
        this.path = path;
    }

    @Override
    public void start() throws Exception {
        this.client = getVertx()
                .createHttpClient();
        // TODO refactor it as it's deprecated
        client.webSocket(port, hostname, path, this::handleMessage);
    }

    private void handleMessage(AsyncResult<WebSocket> webSocketAsyncResult) {
        logger.info("Handling websocket");
        if (!webSocketAsyncResult.succeeded()) {
            logger.warn("Websocket handler connection failed", webSocketAsyncResult.cause());
            try {
                stop();
            } catch (Exception e) {
                logger.error("Failed to stop particle", e);
            }
            return;
        }
        WebSocket webSocket = webSocketAsyncResult.result();
        vertx.setPeriodic(100, new ClientPeriodicStdinScanner(webSocket, System.in));
        webSocket
                .textMessageHandler(msg -> logger.info("Got message: {}", msg))
                .closeHandler(unused -> {
                    try {
                        stop();
                    } catch (Exception e) {
                        logger.error("Got an error while stopping", e);
                    }
                });

    }

    @Override
    public void stop() throws Exception {
        isShuttingDownActivated = true;
        client.close();
        synchronized (runningMonitor) {
            runningMonitor.notifyAll();
        }
    }

    public void waitingUntilFinished() {
        while(!isShuttingDownActivated) {
            synchronized (runningMonitor) {
                try {
                    runningMonitor.wait();
                    return;
                } catch (InterruptedException e) {
                    logger.warn("Got interrupted", e);
                }
            }
        }
    }

}
